use serde::{Deserialize, Serialize};
use std::{
    fs,
    path::PathBuf,
    time::{SystemTime, UNIX_EPOCH},
};

#[derive(Serialize, Deserialize)]
pub struct Cache<T> {
    pub timestamp: u64,
    pub data: T,
}

fn get_cache_path(report_type: &str) -> PathBuf {
    let home = std::env::var("HOME").expect("HOME not set");
    let cache_dir = PathBuf::from(format!("{}/.local/state/trmnl", home));
    let _ = fs::create_dir_all(&cache_dir);
    cache_dir.join(format!("{}.json", report_type))
}

use serde::de::DeserializeOwned;
pub fn load_cache<T: Serialize + DeserializeOwned>(
    report_type: &str,
    expiry_secs: u64,
) -> Option<T> {
    let path = get_cache_path(report_type);
    let contents = fs::read_to_string(&path).ok()?;
    let parsed: Cache<T> = serde_json::from_str(&contents).ok()?;
    let now = SystemTime::now()
        .duration_since(UNIX_EPOCH)
        .unwrap()
        .as_secs();
    if now - parsed.timestamp <= expiry_secs {
        println!("(using cached {} report)", report_type);
        Some(parsed.data)
    } else {
        None
    }
}

pub fn save_cache<T: Serialize>(report_type: &str, data: &T) {
    let path = get_cache_path(report_type);
    let to_save = Cache {
        timestamp: SystemTime::now()
            .duration_since(UNIX_EPOCH)
            .unwrap()
            .as_secs(),
        data: data,
    };
    let json = serde_json::to_string_pretty(&to_save).unwrap();
    let _ = fs::write(path, json);
}
