mod cache;
mod calendar;
mod config;
mod pollen;
mod weather;

use calendar::fetch_next_events;
use chrono::{DateTime, TimeZone, Utc};
use chrono_tz::Tz;
use config::Config;
use pollen::fetch_pollen_api;
use weather::{fetch_weather, geocode_city, WeatherSummary};

#[tokio::main]
async fn main() {
    let config = match Config::load() {
        Ok(cfg) => cfg,
        Err(e) => {
            eprintln!("Failed to load config: {e}");
            std::process::exit(1);
        }
    };

    // Weather - with cache
    let cache_weather_secs = config.cache_weather_secs.unwrap_or(300);
    let weather: Option<WeatherSummary> =
        if let Some(w) = cache::load_cache::<WeatherSummary>("weather", cache_weather_secs) {
            Some(w)
        } else {
            match fetch_weather(&config.location, &config.weather_api_key).await {
                Ok(data) => {
                    cache::save_cache("weather", &data);
                    Some(data)
                }
                Err(e) => {
                    eprintln!("Failed to fetch weather: {e}");
                    None
                }
            }
        };
    let (_lat, _lon) = if weather.is_some() {
        match geocode_city(&config.location, &config.weather_api_key).await {
            Ok((lat, lon)) => (lat, lon),
            Err(_) => (0.0, 0.0),
        }
    } else {
        (0.0, 0.0)
    };
    if let Some(w) = &weather {
        if w.obs_time_unix > 0 {
            match config.timezone.parse::<Tz>() {
                Ok(tz) => {
                    let dt_local = tz.from_utc_datetime(
                        &Utc.timestamp_opt(w.obs_time_unix, 0).unwrap().naive_utc(),
                    );
                    println!(
                        "Weather (as of {}):",
                        dt_local.format("%Y-%m-%d %H:%M:%S %Z")
                    );
                }
                Err(_) => {
                    println!(
                        "Weather (as of unix timestamp {}) (timezone parse error)",
                        w.obs_time_unix
                    );
                }
            }
        } else {
            println!("Weather:");
        }
        println!(" - Temperature: {}", w.temp);
        println!(" - Conditions: {}", w.current_desc);
        println!(" - High: {}, Low: {}", w.high, w.low);
        println!(" - Forecast: {}", w.daily_desc);
    } else {
        println!("Weather: N/A");
    }

    // Pollen - with cache
    let cache_pollen_secs = config.cache_pollen_secs.unwrap_or(300);
    match if let Some(p) = cache::load_cache::<pollen::PollenSummary>("pollen", cache_pollen_secs) {
        Ok(p)
    } else {
        match fetch_pollen_api(&config.pollen_zip).await {
            Ok(data) => {
                cache::save_cache("pollen", &data);
                Ok(data)
            }
            Err(e) => Err(e),
        }
    } {
        Ok(p) => {
            if let Ok(dt) = DateTime::parse_from_rfc3339(&p.forecast_date) {
                match config.timezone.parse::<Tz>() {
                    Ok(tz) => {
                        let local_time = dt.with_timezone(&tz);
                        println!(
                            "\nPollen.com ({})\n  Forecast for {}:",
                            p.location,
                            local_time.format("%Y-%m-%d %H:%M:%S %Z")
                        );
                    }
                    Err(_) => {
                        println!(
                            "\nPollen.com ({})\n  Forecast for {} (timezone parse error):",
                            p.location, p.forecast_date
                        );
                    }
                }
            } else {
                println!(
                    "\nPollen.com ({})\n  Forecast for {}:",
                    p.location, p.forecast_date
                );
            }

            for day in &["Yesterday", "Today", "Tomorrow"] {
                if let Some(period) = p.periods.get(*day) {
                    println!(
                        "    {:9}: {:>4.1} ({})",
                        day,
                        period.index,
                        period.triggers.join(", ")
                    );
                } else {
                    println!("    {:9}: N/A", day);
                }
            }
        }
        Err(e) => {
            println!("\nFailed to fetch pollen (API): {e}");
        }
    }

    // Calendar - with cache
    let cache_calendar_secs = config.cache_calendar_secs.unwrap_or(300);
    println!("\nUpcoming calendar events:");

    match if let Some(events) =
        cache::load_cache::<Vec<calendar::CalendarEventSummary>>("calendar", cache_calendar_secs)
    {
        Ok(events)
    } else {
        match fetch_next_events(
            &config.calendar_url,
            config.calendar_event_count,
            &config.timezone,
        )
        .await
        {
            Ok(data) => {
                cache::save_cache("calendar", &data);
                Ok(data)
            }
            Err(e) => Err(e),
        }
    } {
        Ok(events) if !events.is_empty() => {
            for (i, event) in events.iter().enumerate() {
                let day = event
                    .start
                    .map(|dt| dt.format("%a %Y-%m-%d").to_string())
                    .unwrap_or_else(|| "".to_string());
                let time = if event.all_day {
                    "all day".to_string()
                } else if let Some(dt) = event.start {
                    dt.format("%H:%M").to_string()
                } else {
                    "".to_string()
                };
                println!("{}. {} {:>8}  {}", i + 1, day, time, event.summary);
            }
        }
        _ => println!("No upcoming calendar events found."),
    }

    println!();
    println!("MBTA Transit (placeholder):");
    println!(" - Red Line: Ashmont in 5 min, Braintree in 10 min");
    println!(" - Orange Line: Northbound in 7 min");
    println!(" - Bus 64: Due in 2 min");
    // In the future, fetch real data from an MBTA API here.

    println!("Inbound Shopify packages:");
    let packages = get_shopify_packages(&config);
    for (i, pkg) in packages.iter().enumerate() {
        println!("{}. {}", i + 1, pkg);
    }
}

// No longer needed (handled above)
// fn get_calendar_events(_config: &Config, n: usize) -> Vec<String> { ... }

fn get_shopify_packages(_config: &Config) -> Vec<String> {
    vec![
        "Order #1234: Shipped - Arriving May 13".to_string(),
        "Order #5678: In transit - Arriving May 15".to_string(),
    ]
}
